﻿/*
 *  CommandLink : Automaticaly links to a page within a Silverlight application
 *  or to a Web page. Can also be Event driven (the application can then
 *  decides what to do).
 *  
 * Copyright © 2010 Olivier Dahan - E-Naxos
 * 
 * www.e-naxos.com
 * Dot.Blog
 * 
 * License : Free to use. Supplied as a sample. no support.
 * Can't be sold or distributed without my explicit authorization.
 * Can be modified and use in commercial applications.
 * 
*/ 

using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Browser;
using System.Windows.Controls;
using System.Windows.Media;

namespace CommandLinkDemo
{
    #region Enum navigation type
    /// <summary>
    /// Navigation types
    /// </summary>
    public enum NavigationType
    {
        /// <summary>
        /// 
        /// </summary>
        PageNavigation,
        /// <summary>
        /// 
        /// </summary>
        BrowserNavigation,
        /// <summary>
        /// 
        /// </summary>
        EventDriven
    }
    #endregion

    #region Event arguments
    /// <summary>
    /// Navigation Event arguments
    /// </summary>
    public class NavigationEventArgs : EventArgs
    {
        /// <summary>
        /// Gets or sets the destination.
        /// </summary>
        /// <value>The destination.</value>
        public string Destination { get; set; }
        /// <summary>
        /// Gets or sets the target.
        /// </summary>
        /// <value>The target.</value>
        public string Target { get; set; }
        /// <summary>
        /// Gets or sets the kind of the URI.
        /// </summary>
        /// <value>The kind of the URI.</value>
        public UriKind UriKind { get; set; }
        /// <summary>
        /// Gets or sets the options.
        /// </summary>
        /// <value>The options.</value>
        public string Options { get; set; }
        /// <summary>
        /// Gets or sets the tag.
        /// </summary>
        /// <value>The tag.</value>
        public object Tag { get; set; }
    }
    #endregion


    /// <summary>
    /// Command link
    /// </summary>
    public partial class CommandLink
    {
        #region Ctor
        /// <summary>
        /// Initializes a new instance of the <see cref="CommandLink"/> class.
        /// </summary>
        public CommandLink()
        {
            // Required to initialize variables
            InitializeComponent();
        }
        #endregion

        #region Navigation Event
        /// <summary>
        /// Event fired when navigation mode is set to "event driven".
        /// </summary>
        public EventHandler<NavigationEventArgs> OnNavigate;
        #endregion

        #region Constants
        //agentsmith spellcheck disable
        private const string defOptions = "toolbar=no,location=no,status=no,menubar=no,resizable=yes";
        private const string defTarget = "_blank";
        //agentsmith spellcheck restore 
        #endregion

        #region NavigationMode Property

        /// <summary>
        /// Gets or sets the navigation mode.
        /// </summary>
        /// <value>The navigation mode.</value>

        [Category("Command Link properties")]
        [Description("Navigation mode (Which navigation method will be used)")]
        public NavigationType NavigationMode
        {
            get { return (NavigationType)GetValue(NavigationModeProperty); }
            set { SetValue(NavigationModeProperty, value); }
        }

        /// <summary>
        /// Gets or sets the navigation mode.
        /// </summary>
        public static readonly DependencyProperty NavigationModeProperty =
            DependencyProperty.Register(
                "NavigationMode", typeof(NavigationType), typeof(CommandLink),
                new PropertyMetadata(NavigationType.EventDriven, onNavigationModeChanged));

        private static void onNavigationModeChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        { }

        #endregion

        #region Title Property

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        /// <value>The title.</value>
        [Category("Command Link properties")]
        [Description("First line of text (link title)")]
        public string Title
        {
            get { return (string)GetValue(TitleProperty); }
            set { SetValue(TitleProperty, value); }
        }

        /// <summary>
        ///  Gets or sets the title.
        /// </summary>
        public static readonly DependencyProperty TitleProperty =
            DependencyProperty.Register(
                "Title", typeof(string), typeof(CommandLink),
                new PropertyMetadata("Title", onTitleChanged));

        private static void onTitleChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var cl = d as CommandLink;
            if (cl != null) cl.txtMain.Text = e.NewValue.ToString();
        }

        #endregion

        #region Description Property

        /// <summary>
        /// Gets or sets the description.
        /// </summary>
        /// <value>The description.</value>
        [Category("Command Link properties")]
        [Description("Second line of text (link description)")]
        public string Description
        {
            get { return (string)GetValue(DescriptionProperty); }
            set { SetValue(DescriptionProperty, value); }
        }

        /// <summary>
        /// Gets or sets the description.
        /// </summary>
        public static readonly DependencyProperty DescriptionProperty =
            DependencyProperty.Register(
                "Description", typeof(string), typeof(CommandLink),
                new PropertyMetadata("description", onDescriptionChanged));

        private static void onDescriptionChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var cl = d as CommandLink;
            if (cl != null) cl.txtDescription.Text = e.NewValue.ToString();
        }

        #endregion

        #region NavigateTo Property

        /// <summary>
        /// Gets or sets the address.
        /// </summary>
        /// <value>The navigate to.</value>
        [Category("Command Link properties")]
        [Description("Navigate to Address.")]
        public string NavigateTo
        {
            get { return (string)GetValue(NavigateToProperty); }
            set { SetValue(NavigateToProperty, value); }
        }

        /// <summary>
        /// Gets or sets the address.
        /// </summary>
        public static readonly DependencyProperty NavigateToProperty =
            DependencyProperty.Register(
                "NavigateTo", typeof(string), typeof(CommandLink),
                new PropertyMetadata(null, onNavigateToChanged));

        private static void onNavigateToChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        { }

        #endregion

        #region NavigationUriKind Property

        /// <summary>
        /// Gets or sets the kind of the navigation URI.
        /// </summary>
        /// <value>The kind of the navigation URI.</value>
        [Category("Command Link properties")]
        [Description("Uri Kind")]
        public UriKind NavigationUriKind
        {
            get { return (UriKind)GetValue(NavigationUriKindProperty); }
            set { SetValue(NavigationUriKindProperty, value); }
        }

        /// <summary>
        /// Gets or sets the kind of the navigation URI.
        /// </summary>
        public static readonly DependencyProperty NavigationUriKindProperty =
            DependencyProperty.Register(
                "NavigationUriKind", typeof(UriKind), typeof(CommandLink),
                new PropertyMetadata(UriKind.RelativeOrAbsolute, onNavigationUriKindChanged));

        private static void onNavigationUriKindChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        { }

        #endregion

        #region NavigationTarget Property

        /// <summary>
        /// Gets or sets the navigation target.
        /// </summary>
        /// <value>The navigation target.</value>
        [Category("Command Link properties")]
        [Description("Navigation target (for Web navigation mode only)")]
        public string NavigationTarget
        {
            get { return (string)GetValue(NavigationTargetProperty); }
            set { SetValue(NavigationTargetProperty, value); }
        }

        /// <summary>
        /// Gets or sets the navigation target.
        /// </summary>
        public static readonly DependencyProperty NavigationTargetProperty =
            DependencyProperty.Register(
                "NavigationTarget", typeof(string), typeof(CommandLink),
                new PropertyMetadata(defTarget, onNavigationTargetChanged));

        private static void onNavigationTargetChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        { }

        #endregion

        #region NavigationOption Property

        /// <summary>
        /// Gets or sets the navigation options.
        /// </summary>
        /// <value>The navigation options.</value>
        [Category("Command Link properties")]
        [Description("Navigation options (for Web navigation mode only)")]
        public string NavigationOptions
        {
            get { return (string)GetValue(NavigationOptionsProperty); }
            set { SetValue(NavigationOptionsProperty, value); }
        }

        /// <summary>
        /// Gets or sets the navigation options.
        /// </summary>
        public static readonly DependencyProperty NavigationOptionsProperty =
            DependencyProperty.Register(
                "NavigationOptions", typeof(string), typeof(CommandLink),
                new PropertyMetadata(defOptions, onNavigationOptionChanged));

        private static void onNavigationOptionChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        { }

        #endregion

        #region interface event

        private void txtMain_MouseLeftButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            navigate();
        }

        #endregion

        #region navigation
        private void navigate()
        {
            switch (NavigationMode)
            {
                case NavigationType.PageNavigation:
                    {
                        DependencyObject p = this;
                        do
                        {
                            p = VisualTreeHelper.GetParent(p);
                        } while ((p != null && (!(p is Page))));
                        if (p == null || (!(p is Page))) return;
                        var pp = (Page)p;
                        pp.NavigationService.Navigate(new Uri(NavigateTo, NavigationUriKind));
                    }
                    break;
                case NavigationType.BrowserNavigation:
                    {
                        if (NavigateTo == null) return;
                        HtmlPage.Window.Navigate(new Uri(NavigateTo, NavigationUriKind), NavigationTarget, NavigationOptions);
                    }
                    break;
                case NavigationType.EventDriven:
                    {
                        var n = OnNavigate;
                        if (n == null) return;
                        n(this, new NavigationEventArgs { Destination = NavigateTo, Options = NavigationOptions, Tag = Tag, Target = NavigationTarget, UriKind = NavigationUriKind });
                    }
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }

        #endregion
    }
}